package generators;

import graph.DrawPassesPerHour;

import java.awt.Color;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

import sql.DatabaseRequest;

public class HourOfDayAveragesDisplay extends JFrame {
	
	public HourOfDayAveragesDisplay(String applicationTitle, String chartTitle) {
		CategoryDataset dataset = createDataset();
		JFreeChart chart = createChart(dataset, chartTitle);
				CategoryPlot plot = (CategoryPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.black);
		BarRenderer barRenderer = (BarRenderer)plot.getRenderer();
		barRenderer.setSeriesPaint(0, Color.BLACK);
		ChartPanel chartPanel = new ChartPanel(chart);
		setContentPane(chartPanel);
	}
	
	
	/** * Creates a chart */
	private JFreeChart createChart(CategoryDataset dataset, String title) {
		final JFreeChart chart = ChartFactory.createBarChart(
				"Day averages",         // chart title
				"Hour of day",               // domain axis label
				"Per cent of vehicles per hour",                  // range axis label
				dataset,                  // data
				PlotOrientation.VERTICAL, // orientation
				true,                     // include legend
				true,                     // tooltips?
				false                     // URLs?
				);
		CategoryPlot plot = (CategoryPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.black);
		BarRenderer barRenderer = (BarRenderer)plot.getRenderer();
		barRenderer.setSeriesPaint(0, Color.BLACK);
		return chart;
	}



	/** * Creates a sample dataset */
	private static CategoryDataset createDataset() {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		double sum = 0;
		for (int i = 0; i < TimeVariableGenerator.hourOfDayAverages.length; i++) {
			dataset.setValue((TimeVariableGenerator.hourOfDayAverages[i]*100)/27.43, "HourOfDayAverages", ""+i);
			sum += TimeVariableGenerator.hourOfDayAverages[i];
		}
		System.out.println(sum*100/27.43);
		return dataset;
	}
	
	public static void main(String[] args) {
		HourOfDayAveragesDisplay demo = new HourOfDayAveragesDisplay("Day Averages", "Day Averages");
		demo.pack();
		demo.setVisible(true);
	}
	

}
