package graph;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import javax.swing.JFrame;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.*;
import org.jfree.data.category.*;
import org.jfree.ui.HorizontalAlignment;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.VerticalAlignment;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;



import sql.DatabaseRequest;


public class DrawPassesPerDayOfWeek extends JFrame{

	private static final long serialVersionUID = 1L;
	private JFreeChart chart;

	public JFreeChart getChart() {
		return chart;
	}

	static SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
	static SimpleDateFormat guiSDF = new SimpleDateFormat("yyyy-MM-dd");


	public void start(String chartTitle, String start, String stop, String sqlString){
		CategoryDataset dataset = null;
		try {
			dataset = createDataset(sdf.parse(start).getTime(), sdf.parse(stop).getTime(), sqlString);
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		JFreeChart chart = createChart(dataset, chartTitle);
		ChartPanel chartPanel = new ChartPanel(chart);
		//		chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
		setContentPane(chartPanel);

		String dates = null;
		dates = "From: " + start + " To: " + stop;
		chart.addSubtitle(new TextTitle(dates, new Font("Dialog", Font.PLAIN, 12), Color.black, RectangleEdge.BOTTOM, HorizontalAlignment.CENTER, VerticalAlignment.BOTTOM, RectangleInsets.ZERO_INSETS));

		this.chart=chart;

		//Write to file
		File file = new File("src/pics/passesPerDayOfWeek.png");
		try {
			ChartUtilities.saveChartAsPNG(file, chart, 800, 800);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public DrawPassesPerDayOfWeek(String applicationTitle, String chartTitle) {
		super(applicationTitle);
	}

	/** * Creates a sample dataset */
	private static CategoryDataset createDataset(long start, long stop, String sqlString) {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		int monday = 0;
		int tuesday = 0;
		int wednesday = 0;
		int thursday = 0;
		int friday = 0;
		int saturday = 0;
		int sunday = 0;
		//		SELECT timestamp, name FROM AutoPASS WHERE name = 'Kroppan Bru' OR name = 'Bjrndalen' or name = 'Klett E6' or name = 'Sluppen Bru' or name = 'Leirfossen' or name = 'Vre' ORDER BY timestamp ASC;
		String query = "SELECT timestamp FROM AutoPASS WHERE timestamp < " + sdf.format(new Date(stop)) + " AND timestamp > " + sdf.format(new Date(start)) + sqlString + " ORDER BY timestamp ASC";
		System.out.println(query);
		ResultSet rs = DatabaseRequest.issueQuery(query);
		try {
			while(rs.next()){
				String time = rs.getString(1);
				Date d = null;
				try {
					d = sdf.parse(time);
				} catch (ParseException e) {
					e.printStackTrace();
				}
				Calendar c = new GregorianCalendar();
				c.setTimeInMillis(d.getTime());
				int day = c.get(Calendar.DAY_OF_WEEK);
				System.out.println(day);
				switch (day) {
				case Calendar.MONDAY:
					monday++;
					break;
				case Calendar.TUESDAY:
					tuesday++;
					break;
				case Calendar.WEDNESDAY:
					wednesday++;
					break;
				case Calendar.THURSDAY:
					thursday++;
					break;
				case Calendar.FRIDAY:
					friday++;
					break;
				case Calendar.SATURDAY:
					saturday++;
					break;
				case Calendar.SUNDAY:
					sunday++;
					break;
				default:
					break;
				}
			}

			dataset.addValue(monday, "Days", "Monday");
			dataset.addValue(tuesday, "Days", "Tuesday");
			dataset.addValue(wednesday, "Days", "Wednesday");
			dataset.addValue(thursday, "Days", "Thursday");
			dataset.addValue(friday, "Days", "Friday");
			dataset.addValue(saturday, "Days", "Saturday");
			dataset.addValue(sunday, "Days", "Sunday");
			rs.close();
			return dataset;

		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}



	/** * Creates a chart */

	private JFreeChart createChart(CategoryDataset dataset, String title) {
		final JFreeChart chart = ChartFactory.createBarChart(
				"Passes per day of week",         // chart title
				"Day",               // domain axis label
				"Pass",                  // range axis label
				dataset,                  // data
				PlotOrientation.VERTICAL, // orientation
				true,                     // include legend
				true,                     // tooltips?
				false                     // URLs?
				);
		
		CategoryPlot plot = (CategoryPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.black);
		BarRenderer barRenderer = (BarRenderer)plot.getRenderer();
		barRenderer.setSeriesPaint(0, Color.BLACK);
		barRenderer.setMaximumBarWidth(.05);
		return chart;
	}



	public static void main(String[] args) {
		DrawPassesPerDayOfWeek demo = new DrawPassesPerDayOfWeek("Passes per day of week", "Which operating system are you using?");
		demo.pack();
		demo.setVisible(true);
	}
} 

