package graph;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.AttributedString;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import javax.swing.JFrame;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.PieSectionLabelGenerator;
import org.jfree.chart.plot.PiePlot3D;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.data.general.PieDataset;
import org.jfree.ui.HorizontalAlignment;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.VerticalAlignment;
import org.jfree.util.Rotation;
import sql.DatabaseRequest;

public class UnregisteredPassesPerTollPlaza extends JFrame {


	private static final long serialVersionUID = 1L;
	
	static long firstEntry;
	static long lastEntry;
	static SimpleDateFormat guiSDF = new SimpleDateFormat("yyyy-MM-dd");
	static SimpleDateFormat databaseSDF = new SimpleDateFormat("yyyyMMddHHmmss");
	private JFreeChart chart;
	
	public JFreeChart getChart(){
		return this.chart;
	}
	
	public void start(String start, String stop, String sqlString){
		PieDataset dataset = null;
		try {
			dataset = createDataset(databaseSDF.parse(start).getTime(), databaseSDF.parse(stop).getTime(), sqlString);
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		JFreeChart chart = createChart(dataset);
		ChartPanel chartPanel = new ChartPanel(chart);
		setContentPane(chartPanel);
		String dates = "From: " + guiSDF.format(new Date(firstEntry)) + " To: " + guiSDF.format(new Date(lastEntry));
		chart.addSubtitle(new TextTitle(dates, new Font("Dialog", Font.PLAIN, 12), Color.black, RectangleEdge.BOTTOM, HorizontalAlignment.CENTER, VerticalAlignment.BOTTOM, RectangleInsets.ZERO_INSETS));
		this.chart = chart;
		//save to file
		File file = new File("src/pics/unregisteredPassesPerTollPlaza.png");
		try {
			ChartUtilities.saveChartAsPNG(file, chart, 800, 800);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public UnregisteredPassesPerTollPlaza(String applicationTitle){
		super(applicationTitle);
	}

	private PieDataset createDataset(long start, long stop, String sqlString) {
		DefaultPieDataset  dataset = new DefaultPieDataset ();
		String query = "SELECT name, timestamp FROM `AutoPASS` WHERE hasTag = 0 AND OCRsuccess = 0 AND timestamp > " + databaseSDF.format(new Date(start)) +" AND timestamp < " + databaseSDF.format(new Date(stop)) + sqlString +" ORDER BY AutoPASS.timestamp ASC";
		System.out.println(query);
		ResultSet rs = DatabaseRequest.issueQuery(query);

			Map<String, Integer> map = new HashMap<String, Integer>();
			try {
				int i = 0;
				String tempLastEntry = null;
				while(rs.next()){
					String tollPlaza = rs.getString(1);
					if(i < 1){
						try {
							firstEntry = databaseSDF.parse(rs.getString(2)).getTime();
							i++;
						} catch (ParseException e) {
							e.printStackTrace();
						}
					}
					if(map.containsKey(tollPlaza)){
						int temp = map.get(tollPlaza);
						System.out.println(temp);
						map.put(tollPlaza, temp+1);
						System.out.println("found");
					}
					else{
						map.put(tollPlaza, new Integer(1));
						System.out.println("not found");
					}
					tempLastEntry = rs.getString(2);
				}
				try {
					lastEntry = databaseSDF.parse(tempLastEntry).getTime();
				} catch (ParseException e) {
					e.printStackTrace();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
			java.util.Iterator<Entry<String, Integer>> it = map.entrySet().iterator();
			while(it.hasNext()){
				Map.Entry<String, Integer> pairs = (Map.Entry<String, Integer>)it.next();
				dataset.setValue(pairs.getKey(), pairs.getValue());
			}
			return dataset;
	}

	private JFreeChart createChart(PieDataset dataset) {
		final JFreeChart chart = ChartFactory.createPieChart3D(
				"Unregistered cars per toll plaza",  // chart title
				dataset,                // data
				true,                   // include legend
				true,
				false
				);

		final PiePlot3D plot = (PiePlot3D) chart.getPlot();
		plot.setStartAngle(290);
		plot.setDirection(Rotation.CLOCKWISE);
		plot.setForegroundAlpha(0.5f);
		plot.setNoDataMessage("No data to display");
		plot.setLegendLabelGenerator(new CustomLabelGenerator());
		plot.setLabelGenerator(new CustomLabelGenerator());
		return chart;
	}
	
	static class CustomLabelGenerator implements PieSectionLabelGenerator {

		@Override
		public AttributedString generateAttributedSectionLabel(PieDataset dataset, Comparable key) {
			return new AttributedString(" ");
		}

		@Override
		public String generateSectionLabel(PieDataset dataset, Comparable key) {
		return dataset != null ? key.toString() + " = " +dataset.getValue(key).intValue() : " ";
		}
	}

	public static void main(String[] args) {
		Calendar c = new GregorianCalendar();
		c.set(2013, 0, 0, 0, 0);
//		UnregisteredPassesPerTollPlaza demo = new UnregisteredPassesPerTollPlaza("Comparison", c.getTimeInMillis(), c.getTimeInMillis()+1000*60*60*24L*5);
//		demo.pack();
//		demo.setVisible(true);
//		demo.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}


}
