package gui;

import graph.DrawPassesPerDayOfWeek;
import graph.DrawPassesPerHour;
import graph.TollString;
import graph.UnregisteredPassesPerDay;
import graph.UnregisteredPassesPerTollPlaza;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;

import statistics.Statistics;


public class Gui extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;
	static SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

	String firstTime;
	String lastTime;
	JRadioButton[] radioButtonArray;
	JCheckBox[] checkBoxArray;
	ButtonGroup bg;
	JTextField startString;
	JTextField stopString;
	JTabbedPane tabbedPane;
	JPanel headerPanel;
	JPanel entirePanel;
	JPanel statisticsPanel;

	static JLabel working;

	public static void  setWorking(String text){
		working.setText(text);
	}

	public static JLabel getWorking(){
		return working;
	}


	DrawPassesPerDayOfWeek passesPerDayOfWeek;
	DrawPassesPerHour passesPerHour;
	UnregisteredPassesPerDay passesPerDay;
	UnregisteredPassesPerTollPlaza unregisteredPassesPerTollPlaza;
	JFrame frame = new JFrame("Per Kristian Fjellby MASTER 2013");

	ActionListener CheckBoxListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent ae) {
			if(ae.getActionCommand().equals("c1")){
				try {
					long stop = sdf.parse(startString.getText()).getTime() + 1000*60*60*24*7;
					stopString.setText(sdf.format(new Date(stop)));
				} catch (ParseException e) {
					e.printStackTrace();
				}
			}
			else if(ae.getActionCommand().equals("c2")){
				try {
					long stop = sdf.parse(startString.getText()).getTime() + 1000*60*60*24;
					stopString.setText(sdf.format(new Date(stop)));
				} catch (ParseException e) {
					e.printStackTrace();
				}
			}
			else if(ae.getActionCommand().equals("c3")){
				startString.setText(firstTime);
				try {
					//Probably not enough space in int -> designate as L !
					long stop = sdf.parse(startString.getText()).getTime() + (1000*60*60*24*30L);
					stopString.setText(sdf.format(new Date(stop)));
				} catch (ParseException e) {
					e.printStackTrace();
				}
			}
			else if(ae.getActionCommand().equals("c4")){
				startString.setText(firstTime);
				stopString.setText(lastTime);
			}
		}
	};


	public void initializeGraphs(){
		passesPerHour = new DrawPassesPerHour("Passes per hour", "Passes per hour");
		passesPerHour.initialize();

		passesPerDayOfWeek = new DrawPassesPerDayOfWeek("Passes per day of week", "Passes per day of week");

		passesPerDay = new UnregisteredPassesPerDay("Unregistered passes per day");

		unregisteredPassesPerTollPlaza = new UnregisteredPassesPerTollPlaza("Unregistered passes per toll plaza");
	}


	public Gui(){
		initializeGraphs();
		entirePanel = new JPanel();
		entirePanel.setLayout(new BorderLayout());
		tabbedPane = new JTabbedPane();

		JComponent headerPanel = makeHeaderPanel(passesPerHour.getTollPlazas(), passesPerHour.getStartTime(), passesPerHour.getStopTime());
		entirePanel.add(headerPanel,BorderLayout.NORTH);

		tabbedPane.addTab("Graphs", null, entirePanel, "Graphs");
		tabbedPane.setMnemonicAt(0, KeyEvent.VK_1);



		/*
		 * STATISTICS PANE
		 */
		statisticsPanel = new JPanel();
		makeStatisticsPane();
		tabbedPane.addTab("Statistics", null, statisticsPanel, "Statistics");
		add(tabbedPane, BorderLayout.NORTH);
	}

	protected void makeStatisticsPane(){
		JLabel count = new JLabel("Number of vehicles on record: ");
		JLabel unregisteredCount = new JLabel("Number of unregistered vehicles on record: ");
		JLabel percentageOfHeavyVehicles = new JLabel("Percentage of Heavy Vehicles (>3.5 Tonnes): ");
		JLabel firstDateEntry = new JLabel("First entry date on record: ");
		JLabel lastDateEntry = new JLabel("Last entry date on record: ");
		JLabel money = new JLabel("Total amount of money charged: ");


		statisticsPanel.setLayout(new GridBagLayout());

		Statistics stat = new Statistics();

		JLabel countData = new JLabel(stat.getNumberOfPasses());
		JLabel unregisteredCountData = new JLabel(stat.getNumberOfUnregisteredPasses());
		JLabel percentageOfHeavyVehiclesData = new JLabel(stat.getPercentageOfHeavyVehicles()+ " %");
		JLabel firstDateEntryData = new JLabel(stat.getFirstDateEntry());
		JLabel lastDateEntryData = new JLabel(stat.getLastDateEntry());
		JLabel moneyData = new JLabel(stat.getTotalMoney() + " NOK");

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		statisticsPanel.add(count, constraints);
		constraints.gridx = 1;
		constraints.gridy = 0;
		statisticsPanel.add(countData, constraints);

		constraints.gridx = 0;
		constraints.gridy = 1;
		statisticsPanel.add(unregisteredCount, constraints);
		constraints.gridx = 1;
		constraints.gridy = 1;
		statisticsPanel.add(unregisteredCountData, constraints);

		constraints.gridx = 0;
		constraints.gridy = 2;
		statisticsPanel.add(percentageOfHeavyVehicles, constraints);
		constraints.gridx = 1;
		constraints.gridy = 2;
		statisticsPanel.add(percentageOfHeavyVehiclesData, constraints);

		constraints.gridx = 0;
		constraints.gridy = 3;
		statisticsPanel.add(firstDateEntry, constraints);
		constraints.gridx = 1;
		constraints.gridy = 3;
		statisticsPanel.add(firstDateEntryData, constraints);

		constraints.gridx = 0;
		constraints.gridy = 4;
		statisticsPanel.add(lastDateEntry, constraints);
		constraints.gridx = 1;
		constraints.gridy = 4;
		statisticsPanel.add(lastDateEntryData, constraints);

		constraints.gridx = 0;
		constraints.gridy = 5;
		statisticsPanel.add(money, constraints);
		constraints.gridx = 1;
		constraints.gridy = 5;
		statisticsPanel.add(moneyData, constraints);
	}


	protected JComponent makeHeaderPanel(String[] plazas, String start, String stop){
		firstTime = start;
		lastTime = stop;
		headerPanel = new JPanel();
		GridBagLayout g = new GridBagLayout();
		headerPanel.setLayout(g);

		/*
		 * CHECK BOX LIST PANEL
		 */
		JPanel selectionPanel = new JPanel();
		selectionPanel.setLayout(new BorderLayout());

		JPanel checklistPanel = new JPanel();

		JRadioButton radio;
		radioButtonArray = new JRadioButton[plazas.length];
		for (int i = 0; i < plazas.length; i++) {
			radio = new JRadioButton(plazas[i]);
			radio.setSelected(false);
			radio.setName(plazas[i]);
			radioButtonArray[i] = radio;
			checklistPanel.add(radio);
		}
		selectionPanel.add(checklistPanel, BorderLayout.NORTH);

		/*
		 * TYPE PANEL
		 */

		JPanel typePanel = new JPanel();
		bg = new ButtonGroup();  
		JCheckBox c1 = new JCheckBox("Passes per day of week", false);
		c1.setName("day_of_week");
		c1.addActionListener(CheckBoxListener);
		c1.setActionCommand("c1");
		JCheckBox c2 = new JCheckBox("Passes per hour of day", false);
		c2.setName("hour_of_day");
		c2.addActionListener(CheckBoxListener);
		c2.setActionCommand("c2");
		JCheckBox c3 = new JCheckBox("Unregistered passes per day", false);
		c3.setName("unregistered_per_day");
		c3.addActionListener(CheckBoxListener);
		c3.setActionCommand("c3");
		JCheckBox c4 = new JCheckBox("Unregistered passes per toll plaza", false);
		c4.setName("unregistered_per_plaza");
		c4.addActionListener(CheckBoxListener);
		c4.setActionCommand("c4");

		bg.add(c1);
		bg.add(c2);
		bg.add(c3);
		bg.add(c4);
		c1.setSelected(true);

		checkBoxArray = new JCheckBox[4];
		checkBoxArray[0] = c1;
		checkBoxArray[1] = c2;
		checkBoxArray[2] = c3;
		checkBoxArray[3] = c4;
		typePanel.add(c1);
		typePanel.add(c2);
		typePanel.add(c3);
		typePanel.add(c4);
		selectionPanel.add(typePanel, BorderLayout.SOUTH);

		/*
		 * BUTTON PANEL
		 */
		JPanel buttonPanel = new JPanel();
		JButton clear = new JButton("Clear all");
		clear.addActionListener(this);
		clear.setActionCommand("clear");
		JButton selectAll = new JButton("Select all");
		selectAll.addActionListener(this);
		selectAll.setActionCommand("selectall");

		buttonPanel.add(clear);
		buttonPanel.add(selectAll);

		/*
		 * TIME PANEL
		 */
		JPanel timePanel = new JPanel();
		GridBagLayout timeLayout = new GridBagLayout();
		timePanel.setLayout(timeLayout);
		JLabel startLabel = new JLabel("START TIME");
		JLabel stopLabel = new JLabel("STOP TIME");

		startString = new JTextField(start);
		stopString = new JTextField(stop);

		GridBagConstraints timeLayoutConstraints = new GridBagConstraints();
		timeLayoutConstraints.gridx = 0;
		timeLayoutConstraints.gridy = 0;
		timePanel.add(startLabel, timeLayoutConstraints);
		timeLayoutConstraints.gridx = 0;
		timeLayoutConstraints.gridy = 1;
		timePanel.add(stopLabel, timeLayoutConstraints);
		timeLayoutConstraints.gridx = 1;
		timeLayoutConstraints.gridy = 0;
		timePanel.add(startString, timeLayoutConstraints);
		timeLayoutConstraints.gridx = 1;
		timeLayoutConstraints.gridy = 1;
		timePanel.add(stopString, timeLayoutConstraints);
		/*
		 * REFRESH PANEL
		 */
		GridLayout gl = new GridLayout(2,1);
		JPanel refreshPanel = new JPanel();
		refreshPanel.setLayout(gl);

		JButton refresh = new JButton("Make Graphics");
		refresh.addActionListener(this);
		refresh.setActionCommand("make");
		
		timeLayoutConstraints.gridx = 2;
		timeLayoutConstraints.gridy = 0;
		timeLayoutConstraints.gridheight = 2;
		timePanel.add(refresh, timeLayoutConstraints);

		GridBagConstraints gbc = new GridBagConstraints();

		gbc.gridx = 0;
		gbc.gridy = 0;
		headerPanel.add(selectionPanel, gbc);

		gbc.gridx = 1;
		gbc.gridy = 0;
		headerPanel.add(buttonPanel,gbc);

		gbc.gridx = 2;
		gbc.gridy = 0;
		headerPanel.add(timePanel,gbc);

		gbc.gridx = 3;
		gbc.gridy = 0;
		headerPanel.add(refreshPanel,gbc);
		return headerPanel;
	}

	protected void makePanel(int typeOfChart) {
		TollString.createString(radioButtonArray);
		
		new Worker(frame, entirePanel, headerPanel, typeOfChart, startString.getText(), stopString.getText(), TollString.sqlString).execute();
		
//		ChartPanel chartPanel = new ChartPanel(chart);
//		chartPanel.setSize(new Dimension(1024, 1024));
//		chartPanel.setLayout(new BorderLayout());
//		entirePanel.removeAll();
//		entirePanel.add(headerPanel);
//		entirePanel.add(chartPanel, BorderLayout.SOUTH);
//		entirePanel.validate();
//		frame.pack();
	}

	public void launch(){
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.add(this);
		frame.pack();
		frame.setSize(new Dimension(frame.getWidth(), 700)); //Adjusts panel to components for display
		frame.setResizable(true);
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			Gui gui = new Gui();
			public void run() {
				gui.launch();
			}
		});
	}


	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals("clear")){
			for (int i = 0; i < radioButtonArray.length; i++) {
				radioButtonArray[i].setSelected(false);
			}
			System.out.println("Clear");
		}
		else if(e.getActionCommand().equals("selectall")){
			for (int i = 0; i < radioButtonArray.length; i++) {
				radioButtonArray[i].setSelected(true);
			}
		}
		else if(e.getActionCommand().equals("make")){
			boolean checked = false;
			//Some toll-plazas have to be selected
			for (int i = 0; i < radioButtonArray.length; i++) {
				if(radioButtonArray[i].isSelected()){
					checked = true;
				}
			}
			if(!checked) {
				System.out.println("ELSE");
				JOptionPane.showMessageDialog(frame, "You have to select one or more toll plazas", "ERROR",1);
			}
			else if(bg.getSelection().getActionCommand() != null){
				switch (bg.getSelection().getActionCommand()) {
				case "c1":
					makePanel(1);
					break;
				case "c2":
					makePanel(2);
					break;
				case "c3":
					makePanel(3);
					break;
				case "c4":
					makePanel(4);
					break;
				default:
					break;
				}
			}
		}
	}
}
